##############################################################################
#
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
#
# Copyright (c) Digital Creations.  All rights reserved.
#
# This license has been certified as Open Source(tm).
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
#
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
#
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
#
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
#
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
#
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
#
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
#
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
#
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
#
#
# Disclaimer
#
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
#
#
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
#
##############################################################################
#
# Copyright (c) 2000 Andy McKay
# All rights reserved.
#
# Distributed under the Zope Public License (above)
#
##############################################################################

__version__ = '0.1.3'

from Globals import HTMLFile
from Globals import MessageDialog
from Acquisition import Implicit
from Persistence import Persistent

from OFS.Folder import Folder
from OFS.DTMLDocument import DTMLDocument
from OFS.DTMLMethod import DTMLMethod
from Products.ZCatalog import ZCatalog
from DocumentTemplate.DT_Util import html_quote

from Products.DocBrowser.parse import Parser

import string
import os
import Globals
import sys

manage_addDocBrowserForm = HTMLFile('dtml/DocBrowserAdd', globals())

def manage_addDocBrowser(self, id, title='', dir='', ext='', make_catalog=0, munge_html=0, REQUEST=None):
    ''' Adds a new object '''
    obj = DocBrowser()
    obj.id = id
    obj.title = title
    obj.ext = ext
    obj.make_catalog = make_catalog
    obj.munge_html = munge_html
    obj.dir = dir

    self._setObject(id, obj)
    obj.add_articles()
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

class DocBrowser(Folder):
    meta_type = 'DocBrowser'

    manage_opt = HTMLFile('dtml/DocBrowserOptions', globals())
    manage_help = HTMLFile('dtml/DocBrowserHelp', globals())

    my_options = (
        {'label': 'Options', 'action': 'manage_opt'},
        {'label': 'Help', 'action':'manage_help'},)

    manage_options = Folder.manage_options + my_options

    __ac_permissions__=(
        ('View management screens', ('manage_opt', 'add_articles', 'delete_articles', 'refresh_articles','prepare_msg','manage_editDocBrowser',) ),
        ('Change permissions', ('manage_access',) ), )

    dir = ''
    ext = '.txt'
    make_catalog = 0
    munge_html = 0
    parse_python = 1
    no = []
    yes = []

    def add_articles(self):
        ''' add stuff '''
        self._reset_msg()
        self._add_catalog()
        self._add_index_and_search()
        self._read_dir()
        self._catalog()

    def manage_editDocBrowser(self, dir='', ext='', make_catalog=0, munge_html=0, parse_python=1, REQUEST=None):
        ''' Edits an object '''
        self.ext = ext
        self.make_catalog = make_catalog
        self.munge_html = munge_html
        self.parse_python = parse_python
        self.dir = dir
        return MessageDialog(title = 'Success', message='Object updated, these will not take effect until you do a refresh.', action='./manage_main',)

    def delete_articles(self):
        ''' delete stuff '''
        for i in self.objectIds(['DTML Document', 'Folder']):
            self._delObject(i)
        get_transaction().commit()

    def refresh_articles(self):
        ''' suprisingly useful '''
        self.delete_articles()
        self.add_articles()
        return MessageDialog(title = 'Success', message='Refresh completed. <br><br>%s' % self.prepare_msg, action='./manage_main',)

    def prepare_msg(self):
        ''' prepare msg '''
        msg = 'Success importing <b>%d</b> objects.<br>' % len(self.yes)
        for m in self.yes:
            msg = msg + '<br>%s.' % m

        msg = msg  + '<br><br><b>Failed</b> importing <b>%d</b> objects.<br>' % len(self.no)
        for m in self.no:
            msg = msg + '<br>%s.' % m

        return msg

    def _reset_msg(self):
        ''' to give information back '''
        self.no = []
        self.yes = []

    def _add_folders(self, path):
        ''' creates a folder for path /a/b/c '''
        for p in string.split('/' + path, '/'):
            if p != '':
                try: self.manage_addFolder(p)
                except: pass
                self = self._getOb(p)

        return self

    def _catalog(self):
        ''' catalog '''
        if self.make_catalog == 1:
            cat = self._getOb('catalog')
            objs = self.ZopeFind(self,obj_metatypes=['DTML Document'])
            for obj in objs:
                cat.catalog_object(obj[1], obj[0])

    def _read_dir(self):
        ''' read dir '''
        def _process_file(arg, dirname, names):
            ''' process file in dir '''
            #  set up objects
            base = arg[0]
            obj = base

            if len(dirname) > len(base.dir):
                # means that the current directory is lower than the root
                obj = base._add_folders(string.replace(dirname[len(base.dir):], '\\', '/'))

            for name in names:
                # check extension
                file = os.path.join(dirname, name)
                if name[-len(base.ext):]==base.ext:
                    # read, parse, add
                    print "Parsing ", file
                    try:
                        data = base._read_file(file)
                        data = base._parse_doc(data)
                        base._add_doc(obj, data, name)
                        base.yes.append(file)
                    except:
                        base.no.append('%s [<b>Error</b> %s]'  % (file, sys.exc_value))
                else:
                    print "Not parsing ", file

        os.path.walk(self.dir, _process_file, (self,))

    def _read_file(self, filename):
        ''' reads file '''
        file = open(filename, 'rb')
        data = file.read()
        file.close()
        return data

    def _parse_doc(self, data):
        ''' parses the doc '''
        if self.munge_html == 1:
            data = html_quote(str(data))
            data = string.replace(data, '\n', '<br>\n')

        if self.parse_python == 1:
            p = Parser(data)
            p.format()
            data = p.out
            del p

        data ='''
        <dtml-call \"RESPONSE.setHeader('Content-type', 'text/html')\">
        <dtml-var standard_html_header>''' + data + '''
        <dtml-var standard_html_footer>
        '''
        return data

    def _add_doc(self, sub_folder, data, id):
        ''' add doc '''
        id = '~' + id
        obj = DTMLDocument(data, __name__ = id)
        obj.title = id[1:]
        sub_folder._setObject(id, obj)

    def _add_index_and_search(self):
        ''' adds in search and index page '''
        if self.make_catalog == 1:
            self._add_page('DocBrowserIndex.dtml', 'index_html')
            self._add_page('DocBrowserSearch.dtml', 'search_html')

    def _add_page(self, file, name):
        if name not in self.objectIds():
            data = self._read_file(Globals.package_home(globals()) + os.sep + 'dtml' + os.sep + file)
            obj = DTMLMethod(data, __name__ = name)
            self._setObject(name, obj)
        else:
            print '%s already exists' % name

    def _add_catalog(self):
        ''' adds a catalog '''
        if self.make_catalog == 1:
            name = 'catalog'
            if name not in self.objectIds():
                obj = ZCatalog.ZCatalog(name)
                self._setObject(name, obj)
            else:
                print '%s already exists' % name