"""
    MoinMoin - Python Source Parser
"""

# Imports
import cgi, string, sys, cStringIO
import keyword, token, tokenize


#############################################################################
### Python Source Parser (does Hilighting)
#############################################################################

_KEYWORD = token.NT_OFFSET + 1
_TEXT    = token.NT_OFFSET + 2

_colors = {
    token.NUMBER:       'number',
    token.OP:           'operator',
    token.STRING:       'string',
    tokenize.COMMENT:   'comment',
    token.NAME:         'name',
    token.ERRORTOKEN:   'error',
    _KEYWORD:           'keyword',
    _TEXT:              'text',
}


class Parser:
    """ Send colored python source.
    """

    def __init__(self, raw):
        """ Store the source text.
        """
        self.raw = string.strip(string.expandtabs(raw))
        self.out = ''
        self.line_nos = {}
        self.found_license = 0

    def format(self, formatter=None, form=None, license=None):
        """ Parse and send the colored source.
        """
        if string.find(self.raw, '# Zope Public License (ZPL) Version 1.0') >= 0:
            length = 78
            hashes = '#' * length
            end = string.find(self.raw[length:], hashes)
            self.raw = self.raw[end+length*2:]
            self.found_license = 1

        # store line offsets in self.lines
        self.lines = [0, 0]
        pos = 0
        while 1:
            pos = string.find(self.raw, '\n', pos) + 1
            if not pos: break
            self.lines.append(pos)
        self.lines.append(len(self.raw))

        # parse the source and write it
        self.pos = 0
        text = cStringIO.StringIO(self.raw)

        try:
            tokenize.tokenize(text.readline, self)
        except tokenize.TokenError, ex:
            msg = ex[0]
            line = ex[1][0]
            self.out = self.out + "<b>ERROR: %s</b>%s\n" % (msg, self.raw[self.lines[line]:])

        self.add_line_numbers()
        self.add_header()
        self.add_footer()

    def add_header(self):
        h = ''
        if self.found_license:
                h = h + '<dtml-if theZopeLicense><dtml-var theZopeLicense></dtml-if>\n'
        h = h + '<pre>\n'

        self.out = h + self.out

    def add_footer(self):
        self.out = self.out + '\n</pre>'

    def add_line_numbers(self):
        in_lines = string.split(self.out, '\n')
        out_lines = []
        line_no = 1
        if self.found_license:
            line_no = line_no + 84
            in_lines = in_lines[1:]
        for line in in_lines:
            spaces = ' ' * ((4 - len(str(line_no))) + 4)
            out_lines.append(('<a name="%s">%s</a>%s' % (line_no, line_no, spaces)) + line)
            line_no = line_no + 1

        self.out = string.join(out_lines, '\n')

    def __call__(self, toktype, toktext, (srow,scol), (erow,ecol), line):
        """ Token handler.
        """
        # calculate new positions
        oldpos = self.pos
        newpos = self.lines[srow] + scol
        self.pos = newpos + len(toktext)

        # handle newlines
        if toktype in [token.NEWLINE, tokenize.NL]:
            self.out = self.out + '\n'
            return

        # send the original whitespace, if needed
        if newpos > oldpos:
            self.out = self.out + (self.raw[oldpos:newpos])

        # skip indenting tokens
        if toktype in [token.INDENT, token.DEDENT]:
            self.pos = newpos
            return

        # map token type to a color group
        if token.LPAR <= toktype and toktype <= token.OP:
            toktype = token.OP
        elif toktype == token.NAME and keyword.iskeyword(toktext):
            toktype = _KEYWORD
        color = _colors.get(toktype, _colors[_TEXT])

        style = ''
        if toktype == token.ERRORTOKEN:
            style = ' style="border: solid 1.5pt #FF0000;"'

        # send text
        if toktype == tokenize.COMMENT:
            toktext = string.strip(toktext)

        self.out = self.out + '<span class="%s"%s>' % (color, style)

        link_flag = 0
        if toktype == token.NAME:
            words = string.split(line, ' ')
            keywords = ['from', 'import']
            for k in keywords:
                if k in words:
                    link_flag = 1

        if link_flag:
            self.out = self.out + '<a href="search_html?PrincipiaSearchSource=%s">%s</a>' % (cgi.escape(toktext), cgi.escape(toktext))
        else:
            self.out = self.out + cgi.escape(toktext)
        self.out = self.out + '</span>'

        if toktype == tokenize.COMMENT:
            self.out = self.out + '\n'

def read(arg, dirname, files):
    for file in files:
        if file[-3:] == '.py':
            infile = os.path.join(dirname, file)
            outfile = os.path.join(dirname, file + '.out')
            colour(infile, outfile)

def colour(infile, outfile):
    license = open('license.txt', 'r').read()
    p = Parser(open(infile, 'r').read())
    p.format(license=license)

    out = open(outfile, 'w')
    out.write(p.out)
    out.close()

if __name__ == "__main__":
    import os, sys
    os.path.walk('e:\\Zope-2.3.1-src', read, None)

